//  Type MKDD *.DBF  to produce DATADICT.DPR and DDIMPLIB.PAS
//
//
// John Wright 100335,322

#include "directry.ch"
#include "fileio.ch"

#define FIELD_REC                          32
#define FIELD_LEN                          10

// Statics are really globals<G>

static nDLLHandle := -1
static nIMPHandle := -1

FUNCTION Main(cFileSpec)

  // Validate command line
  
  if cFileSpec == Nil
    Alert("Please supply a filename.", {"OK"})
  else  

    // Process all DBF files to Units

    GenUnits(cFileSpec)
  endif
    
RETURN NIL


static function GenUnits(cFileSpec)
  local aFileSpec := Directory(cFileSpec)
  Local cFileName := ''
  Local i := 0
  Local cTerminator := ''
  
  // Create DPR for making DLL and PAS for the import unit

  nDLLHandle := fcreate("DATADICT.DPR")
  nIMPHandle := fcreate("DDIMPLIB.PAS")
  if nDLLHandle != -1
    if nIMPHandle != -1

      // Write file headers

      DLLDumpHeader()
      IMPDumpHeader()

      // first pass - the code

      for i := 1 to len(aFileSpec)
        cFileName := fparsefn(aFileSpec[i][F_NAME])
        if right(aFileSpec[i][F_NAME], 3) == 'DBF' .and. file(cFileName+'.DBF')
          GenUnit(cFileName)
        endif
      next i

      // second pass - the exports and implementation sections

      fDLLBlankLine()
      fDLLWriteLine("Exports")

      fIMPBlankLIne()
      fIMPWriteLine("Implementation")
      fIMPBlankLine()

      for i := 1 to len(aFileSpec)
        cFileName := fparsefn(aFileSpec[i][F_NAME])
        if right(aFileSpec[i][F_NAME], 3) == 'DBF' .and. file(cFileName+'.DBF')
          if i == len(aFileSpec)
            cTerminator := ';'
          else
            cTerminator := ','
          endif
          fDLLWriteLine("  DD"+upper(cFileName)+" index "+alltrim(str(i))+cTerminator)
          fIMPWriteLine("  function DD"+upper(cFileName)+"; external 'DATADICT' index "+alltrim(str(i))+" ;")

        endif
      next i
      fDLLBlankLine()
      DLLDumpFooter()
      ImpDumpFooter()
      fclose(nIMPHandle)
    endif
    fclose(nDLLHandle)
  endif

return nil


static function GenUnit(cFileName)
  Local i := 0
  Local aStruct := {}
  Local nFields := 0
  Local cFieldName := ''
  
  // Create a pascal unit

  ? " Processing : "+cFileName

  // Get the structure of the DBF file without using dbusearea/dbclosearea
  // this keeps the size of the EXE down

  aStruct := hDBStruct(cFileName)
  if !empty(aStruct)

    nFields := len(aStruct)

    // Capitalize filename

    cFileName := lower(cFileName)
    cFileName := upper(left(cFileName, 1))+right(cFilename, len(cFileName)-1)

    fIMPWriteLine("Function DD"+upper(cFileName)+"(sName:string) :integer ;")

    fDLLBlankLine()
    fDLLWriteLine("Function DD"+upper(cFileName)+"(sName : string) : integer ; export;")
    fDLLWriteLine("Begin")
    fDLLBlankLine()
    fDLLWriteLine("  Result := 0 ;")
    fDLLBlankLine()
    fDLLWriteLine("  sName := sName+#0 ;")
    fDLLBlankLine()
    fDLLWriteLine("  if sx_createnew(@sName[1], 'TMPALIAS', SIXFOX, "+alltrim(str(nFields))+") <> 0 then")
    fDLLWriteLine("  Begin")


    for i := 1 to nFields
      cFieldName := Padr("'"+aStruct[i][1]+"',", 15)
      fDLLWriteLine("    sx_createfield("+cFieldName+"'"+;
                                       aStruct[i][2]+"',"+;
                                       alltrim(str(aStruct[i][3]))+","+;
                                       alltrim(str(aStruct[i][4]))+") ;")
    next i

    fDLLBlankLine()
    fDLLWriteLine("    if sx_createexec then")
    fDLLWriteLine("    Begin")
    fDLLWriteLine("      sx_close ; ")
    fDLLWriteLine("      Result := 1 ;")
    fDLLWriteLine("    End ; ")
    fDLLWriteLine("  end ;")
    fDLLBlankLine()
    fDLLWriteLine("End;")
    fDLLBlankLine()

  endif
return nil


static function DLLDumpHeader()

  fDLLWriteLine("Library DATADICT ;" )
  fDLLBlankLine()
  fDLLWriteLine("Uses")
  fDLLWriteLine("  Rocket ;")
  fDLLBlankLine()

return nil

static function DLLDumpFooter()
  fDLLWriteLine("Begin")
  fDLLBlankLine()
  fDLLWriteLine("  { Call the Rocket set string type function to ensure that we }")
  fDLLWriteLine("  { always use C style strings }")
  fDLLBlankLine()
  fDLLWriteLine("  sx_setstringtype(1) ;")
  fDLLBlankLine()
  fDLLWriteLine("End.")
return nil

static function fDLLBlankLine()
  fDLLWriteLine("")
return nil

static function fDLLWriteLine(cString)
return fWrite(nDLLHandle, cString+chr(13)+chr(10))


static function IMPDumpHeader()

  fIMPWriteLine("Unit DDIMPLIB ;" )
  fIMPBlankLine()
  fIMPWriteLine("Interface")
  fIMPBlankLine()

return nil

static function IMPDumpFooter()
  fIMPWriteLine("End.")
return nil

static function fIMPBlankLine()
  fIMPWriteLine("")
return nil

static function fIMPWriteLine(cString)
return fWrite(nIMPHandle, cString+chr(13)+chr(10))


// I'd love to use Funcky but lets keep it simple...
static function fParsefn(sPath)
  Local nPos := AT('.', sPath)
  Local cFileName := ''

  if nPos > 0
    cFileName := substr(sPath, 1, nPos-1)
  else
    cFileName := sPath
  endif
return cFileName

// This is based upon some PD code code from the CLIPPER forum.
// If I knew who it was I'd give them all due credit.
static function hDBStruct(cFileName)
  local aStruct := {}
  local nIMPHandle := 0
  local nField  := 1
  local cField  := ''
  local cType   := ''
  local nLen    := 0
  Local nDec    := 0
  Local cPath   := ''
  Local cTable  := ''
  
  if right(cFileName, 4) != '.DBF'
    cFileName += '.DBF'
  endif

  nIMPHandle := fopen(cFileName, FO_READ + FO_SHARED )
  if nIMPHandle != -1
  
    while .t.
      fseek( nIMPHandle, FIELD_REC * nField++, FS_SET )
      if freadStr( nIMPHandle, 1 ) == chr(13)
        fclose(nIMPHandle)
        exit
      else
        fseek( nIMPHandle, -1, FS_RELATIVE )
      endif
    
      // name of field
      
      cField := padr(freadStr( nIMPHandle, FIELD_LEN ),FIELD_LEN)
    
      fseek( nIMPHandle, 1,FS_RELATIVE )
    
      // type of field C/N/D/L/M
      
      cType  := freadStr( nIMPHandle, 1 )
      fseek( nIMPHandle, -1,FS_RELATIVE )
    
      fseek( nIMPHandle, 5,FS_RELATIVE )
    
      // length of current field
      
      nLen   := asc(freadStr( nIMPHandle, 1 ))
      fseek( nIMPHandle, -5,FS_RELATIVE )
    
      fseek( nIMPHandle, 5,FS_RELATIVE )
      
      // number decimals
      
      if cType == "N"
        nDec := asc(freadStr( nIMPHandle, 1 ) )
      else
        nDec := 0
      endif
    
      fseek( nIMPHandle, -5,FS_RELATIVE )
    
      aadd( aStruct, {alltrim(cField),cType,nLen,nDec} )
    enddo
  else
    return {}
  endif
return aStruct
